# -*- coding: utf-8 -*-
"""
Created on Wed May 15 22:50:12 2019

@author: MAndrle
"""

## Sample-Splitting with Lasso
"""
This code illustrates inference after variable selection using 
"sample splitting"

The sample is randomly split into two parts, S1 and S2.
1) Using S1 we carry out variable selection, e.g. using lasso
2) Using S2 we do OLS using the selected variables and standard inference...

"""
## imports of modules
import numpy as np
import pandas as pd 

from sklearn.linear_model import LinearRegression
from sklearn.linear_model import LassoCV

import matplotlib.pyplot as plt
import seaborn as sns # let's try to use seaborn, as long as it creates valid matplotlibs



## Simulation setup
k = 20
N = 300


## Data-Generating Process
def dgp():
    ix_true = np.array([1, 3, 10, 15])
    beta_val = np.array([0.7, 0.2, 0.5, 1]).reshape(4,1)
    
    # Cross-correlation will make it quite difficult for LASSO....
    SIG = np.zeros([k,k])
    for ix_r in np.arange(k):
        for ix_c in np.arange(k):
            SIG[ix_r, ix_c] = 0.5 **np.absolute(ix_r - ix_c)

    # Sample the X explanatory variables 
    X = np.random.multivariate_normal(np.zeros(k), SIG, size=[N,])
    
    # create the true model
    #Y = X[:, ix_true] @ beta_val + np.random.normal(0, 1, size=[N,1])
    beta = np.random.normal(0,.02,size=[k,1])
    beta[ix_true,] = beta_val
    
    Y = X @ beta + np.random.normal(0, 1, size=[N,1])
    
    return (Y, X, ix_true, beta)    
    
   


## Main code


ndraws = 5000

theta_est = np.zeros([ndraws, k])

for s in np.arange(0, ndraws):
    
    # keeping track of simulations
    if ((s % 100) == 0):
       print('Iteration #: %d ' % s)
    
    # draw from the true model
    # ---------------------------------
    Y, X, ix_true, beta = dgp()
    
    # split the sample
    # ---------------------------------
    train_portion = 0.5  
    
    all_ix   = np.arange(N)
    ix_train = np.random.choice(N,np.int(train_portion * N), replace=False) # resample indices
    ix_train.sort() # just for the fun of it...
    ix_test  = np.setdiff1d(all_ix, ix_train)  # set difference, to find 'test'
    
    X_train = X[ix_train, ]
    X_test  = X[ix_test,  ]
    
    Y_train = Y[ix_train, ]
    Y_test  = Y[ix_test,  ]
    
    # train cross-validated LASSO on X_train
    # -----------------------------------------------------------------
    #ML = LassoCV(cv=5, random_state=0, max_iter=200, tol=0.01)
    ML = LassoCV(cv=3, random_state=0)
    ML.fit(X_train, Y_train.flatten())
    
    lasso_coefs = ML.coef_ 
    ix_valid = lasso_coefs > 0.01
    
    # use the model selected by LASSO for X_test
    # -----------------------------------------------------------------
    X_ols = X_test[:, ix_valid]

    if (np.sum(ix_valid) > 0):  # make sure some regressors were picked...
        final_ols = LinearRegression()
        final_ols.fit(X_ols, Y_test.flatten())
    
        theta_est[s, ix_valid] = final_ols.coef_
   
    
 
    
## Process the Results    
# ----------------------------------------------------
beta_true = np.zeros([k,1])
beta_true[ix_true, :] = beta[ix_true,:]

# Plot the figure to see the distributions
sns.set_style("white")
plt.figure(figsize=(15,15))
for i in np.arange(k):

    ax = plt.subplot(4, 5, i+1)

    #pp = plt.hist(theta_est[:, i])
    sns_plot = sns.distplot(theta_est[:, i], hist=False);
    plt.axvline(beta_true[i], 0, 4, color='red', linewidth=1)
    
    sns_plot.spines['right'].set_visible(False)
    sns_plot.spines['top'].set_visible(False)


#sns_plot.figure.savefig("lasso_sample_split.eps")












